from .rocker_util import RockerControlUtil


def transform_both_speeds(linear: float, angular: float, linear_percent: float, angular_percent: float):
    transfrom3 = RockerControlUtil.getInstance().transfrom3(linear, linear_percent)
    transform4 = RockerControlUtil.getInstance().transfrom3(angular, angular_percent)

    if transfrom3 is not None and len(transfrom3) > 0:
        linear_speed = transfrom3[0] * 10
        angular_speed = int(transform4[1] * 4.5)
        return linear_speed, angular_speed

def get_percent(percent: float):
    if percent <= 15.0:
        return 0.0

    return percent - 15.0


from dataclasses import dataclass
from enum import Enum

from pymammotion.utility.conversions import parse_double


class RTKStatus(Enum):
    NONE = 0
    BAD = 1
    FINE = 4


@dataclass
class RapidState:
    pos_x: float = 0
    pos_y: float = 0
    rtk_status: RTKStatus = RTKStatus.NONE
    toward: float = 0
    satellites_total: int = 0
    satellites_l2: int = 0
    rtk_age: float = 0
    lat_std: float = 0
    lon_std: float = 0
    pos_type: int = 0
    zone_hash: int = 0
    pos_level: int = 0

    @classmethod
    def from_raw(cls, raw: list[int]) -> "RapidState":
        return RapidState(
            rtk_status=RTKStatus.FINE if raw[0] == 4 else RTKStatus.BAD if raw[0] in (1, 5) else RTKStatus.NONE,
            pos_level=raw[1],
            satellites_total=raw[2],
            rtk_age=parse_double(raw[3], 4.0),
            lat_std=parse_double(raw[4], 4.0),
            lon_std=parse_double(raw[5], 4.0),
            satellites_l2=raw[6],
            pos_x=parse_double(raw[7], 4.0),
            pos_y=parse_double(raw[8], 4.0),
            toward=parse_double(raw[9], 4.0),
            pos_type=raw[10],
            zone_hash=raw[11],
        )



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: pymammotion/proto/luba_mul.proto
# plugin: python-betterproto
from dataclasses import dataclass

import betterproto


class MUL_LANGUAGE(betterproto.Enum):
    ENGLISH = 0
    GERMAN = 1


class MUL_CAMERA_POSITION(betterproto.Enum):
    LEFT = 0
    RIGHT = 1
    REAR = 2
    ALL = 3


class MUL_VIDEO_ERROR_CODE(betterproto.Enum):
    SUCCESS = 0
    ACTIVATION_FAILED = 1
    NETWORK_NOT_AVAILABLE = 2
    CREATE_CHANNEL_FAILED = 3
    PARAM_INVAILD = 4
    CELLULAR_RESTRICTION = 5


class MUL_WIPER_ERROR_CODE(betterproto.Enum):
    SET_SUCCESS = 0
    HW_ERROR = 1
    NAVIGATION_WORK_FORBID = 2


@dataclass
class MulSetAudio(betterproto.Message):
    at_switch: int = betterproto.int32_field(1, group="AudioCfg_u")
    au_language: "MUL_LANGUAGE" = betterproto.enum_field(2, group="AudioCfg_u")


@dataclass
class MulSetVideo(betterproto.Message):
    position: "MUL_CAMERA_POSITION" = betterproto.enum_field(1)
    vi_switch: int = betterproto.int32_field(2)


@dataclass
class MulSetVideoAck(betterproto.Message):
    error_code: "MUL_VIDEO_ERROR_CODE" = betterproto.enum_field(1)


@dataclass
class MulAudioCfg(betterproto.Message):
    au_switch: int = betterproto.int32_field(1)
    au_language: "MUL_LANGUAGE" = betterproto.enum_field(2)


@dataclass
class MulSetWiper(betterproto.Message):
    round: int = betterproto.int32_field(1)


@dataclass
class MulSetWiperAck(betterproto.Message):
    error_code: "MUL_WIPER_ERROR_CODE" = betterproto.enum_field(1)


@dataclass
class SocMul(betterproto.Message):
    set_audio: "MulSetAudio" = betterproto.message_field(1, group="SubMul")
    audio_cfg: "MulAudioCfg" = betterproto.message_field(2, group="SubMul")
    set_video: "MulSetVideo" = betterproto.message_field(3, group="SubMul")
    set_video_ack: "MulSetVideoAck" = betterproto.message_field(4, group="SubMul")
    set_wiper: "MulSetWiper" = betterproto.message_field(5, group="SubMul")
    set_wiper_ack: "MulSetWiperAck" = betterproto.message_field(6, group="SubMul")



import logging
from typing import List

logger = logging.getLogger(__name__)


class GenerateRouteInformation:
    """Creates a model for generating route information and mowing plan before starting a job."""

    def __init__(
        self,
        one_hashs: List[int],
        job_mode: int,
        channel_width: int,
        speed: float,
        ultra_wave: int,
        channel_mode: int,
        rain_tactics: int,
        toward: int,
        knife_height: int,
        path_order: str,
        toward_included_angle: int,
    ):
        self.path_order = ""
        self.toward_mode = 0
        self.one_hashs = one_hashs
        self.rain_tactics = rain_tactics
        self.job_mode = job_mode
        self.knife_height = knife_height
        self.speed = speed
        self.ultra_wave = ultra_wave
        self.channel_width = channel_width
        self.channel_mode = channel_mode
        self.toward = toward
        self.edge_mode = rain_tactics
        self.path_order = path_order
        self.toward_included_angle = toward_included_angle
        logger.debug(
            f"Mode route command parameters jobMode={job_mode}//channelWidth={channel_width}//speed={speed}//UltraWave={ultra_wave}//channelMode={channel_mode}//edgeMode={rain_tactics}//knifeHeight={knife_height}  pathOrder:{path_order.encode('utf-8')}"
        )

    def get_job_id(self) -> int:
        return self.job_id

    def set_job_id(self, job_id: int) -> None:
        self.job_id = job_id

    def get_job_ver(self) -> int:
        return self.job_ver

    def set_job_ver(self, job_ver: int) -> None:
        self.job_ver = job_ver

    def get_rain_tactics(self) -> int:
        return self.rain_tactics

    def set_rain_tactics(self, rain_tactics: int) -> None:
        self.rain_tactics = rain_tactics

    def get_job_mode(self) -> int:
        return self.job_mode

    def set_job_mode(self, job_mode: int) -> None:
        self.job_mode = job_mode

    def get_knife_height(self) -> int:
        return self.knife_height

    def set_knife_height(self, knife_height: int) -> None:
        self.knife_height = knife_height

    def get_speed(self) -> float:
        return self.speed

    def set_speed(self, speed: float) -> None:
        self.speed = speed

    def get_ultra_wave(self) -> int:
        return self.ultra_wave

    def set_ultra_wave(self, ultra_wave: int) -> None:
        self.ultra_wave = ultra_wave

    def get_channel_width(self) -> int:
        return self.channel_width

    def set_channel_width(self, channel_width: int) -> None:
        self.channel_width = channel_width

    def get_channel_mode(self) -> int:
        return self.channel_mode

    def set_channel_mode(self, channel_mode: int) -> None:
        self.channel_mode = channel_mode

    def get_toward(self) -> int:
        return self.toward

    def set_toward(self, toward: int) -> None:
        self.toward = toward

    def get_one_hashs(self) -> List[int]:
        return self.one_hashs if self.one_hashs else []

    def set_one_hashs(self, one_hashs: List[int]) -> None:
        self.one_hashs = one_hashs

    def get_path_order(self) -> str:
        return self.path_order

    def set_path_order(self, path_order: str) -> None:
        self.path_order = path_order

    def get_toward_included_angle(self) -> int:
        return self.toward_included_angle

    def set_toward_included_angle(self, toward_included_angle: int) -> None:
        self.toward_included_angle = toward_included_angle

    def get_toward_mode(self) -> int:
        return self.toward_mode

    def get_edge_mode(self) -> int:
        return self.edge_mode

    def set_edge_mode(self, edge_mode: int) -> None:
        self.edge_mode = edge_mode

    def __str__(self) -> str:
        try:
            return f"GenerateRouteInformation{{oneHashs={self.one_hashs}, jobId={self.job_id}, jobVer={self.job_ver}, rainTactics={self.rain_tactics}, jobMode={self.job_mode}, knifeHeight={self.knife_height}, speed={self.speed}, UltraWave={self.ultra_wave}, channelWidth={self.channel_width}, channelMode={self.channel_mode}, toward={self.toward}, pathOrder='{self.path_order.encode('utf-8')}', edgeMode={self.edge_mode}, towardIncludedAngle={self.toward_included_angle}}}"
        except Exception as e:
            return str(e)



from dataclasses import dataclass
from typing import List, Optional

from mashumaro.config import BaseConfig
from mashumaro.mixins.orjson import DataClassORJSONMixin


@dataclass
class Device(DataClassORJSONMixin):
    gmtModified: int
    netType: str
    categoryKey: str
    productKey: str
    nodeType: str
    isEdgeGateway: bool
    deviceName: str
    categoryName: str
    identityAlias: str
    productName: str
    iotId: str
    bindTime: int
    owned: int
    identityId: str
    thingType: str
    status: int
    nickName: Optional[str] = None
    description: Optional[str] = None
    productImage: Optional[str] = None
    categoryImage: Optional[str] = None
    productModel: Optional[str] = None


    class Config(BaseConfig):
        omit_default = True


@dataclass
class Data(DataClassORJSONMixin):
    total: int
    data: List[Device]
    pageNo: int
    pageSize: int


@dataclass
class ListingDevByAccountResponse(DataClassORJSONMixin):
    code: int
    data: Optional[Data]
    id: Optional[str] = None



import copy
import math


class RockerControlUtil:
    """generated source for class RockerControlUtil"""

    instance_ = None
    list_ = []
    thresholdValue_1 = 30
    thresholdValue_2 = 7
    thresholdValue_3 = 15

    def __init__(self):
        """Generated source for method __init__"""

    @classmethod
    def getInstance(cls):
        """Return the instance of RockerControlUtil if it exists, otherwise create
        a new instance.

        This method checks if an instance of RockerControlUtil exists. If not,
        it creates a new instance and returns it.

        Args:
            cls (class): The class for which the instance is being retrieved.

        Returns:
            RockerControlUtil: An instance of RockerControlUtil.

        """
        if cls.instance_ == None:
            cls.instance_ = RockerControlUtil()
        return cls.instance_

    def transfrom(self, f, f2):
        """Perform a transformation based on the input angles and distance.

        This method calculates the transformation based on the input angle 'f'
        and distance 'f2'. It determines the appropriate radians based on the
        angle and performs the transformation.

        Args:
            f (float): The angle in degrees for transformation.
            f2 (float): The distance for transformation.

        Returns:
            list: A list containing the transformed coordinates.

        """
        radians = 0.0
        self.list_.clear()
        i = self.thresholdValue_2
        if f > 90 - i and f < i + 90:
            radians = math.radians(90.0)
        elif f > 270 - i and f < i + 270:
            radians = math.radians(270.0)
        elif f > 180 - i and f < i + 180:
            radians = math.radians(180.0)
        elif f < i and f > 360 - i:
            radians = math.radians(0.0)
        else:
            if f > i:
                i2 = self.thresholdValue_2
                if f < 90 - i2:
                    radians = math.radians(90 - i2)

            i3 = self.thresholdValue_1
            if f > i3 + 90 and f < 180 - i:
                radians = math.radians(i3 + 90)
            elif f > i + 180 and f < 270 - i3:
                radians = math.radians(270 - i3)
            elif f > i3 + 270 and f < 360 - i:
                radians = math.radians(i3 + 270)
            else:
                radians = math.radians(f)
        d = f2
        self.list_.append(int(math.sin(radians) * d))
        self.list_.append(int(d * math.cos(radians)))
        return copy.copy(self.list_)

    def transfrom2(self, f, f2):
        """Calculate the transformation of input angles to radians and perform
        trigonometric calculations.

        This method takes two input parameters, an angle 'f' and a value 'f2',
        and calculates the corresponding radians based on the angle. It then
        performs trigonometric calculations using the radians and 'f2' value to
        generate a list of transformed values.

        Args:
            self: The instance of the class.
            f (float): The input angle in degrees.
            f2 (float): The input value for trigonometric calculations.

        Returns:
            list: A list containing the transformed values based on trigonometric
                calculations.

        """
        radians = 0.0
        self.list_.clear()
        i = self.thresholdValue_2
        if f > 90 - i and f < i + 90:
            radians = math.radians(90.0)
        elif f > 270 - i and f < i + 270:
            radians = math.radians(270.0)
        else:
            i2 = self.thresholdValue_3
            if f > 180 - i2 and f < i2 + 180:
                radians = math.radians(0.0)
            elif f < i2 or f > 360 - i2:
                radians = math.radians(180.0)
            else:
                if f > i2:
                    i3 = self.thresholdValue_1
                    if f < 90 - i3:
                        radians = math.radians(i3 + 90)
                i4 = self.thresholdValue_1
                if f > i4 + 90 and f < 180 - i2:
                    radians = math.radians(90 - i4)
                elif f > i2 + 180 and f < 270 - i4:
                    radians = math.radians(i4 + 270)
                elif f > i4 + 270 and f < 360 - i2:
                    radians = math.radians(270 - i4)
                elif f > 270 - i4 and f < 270 - i:
                    radians = math.radians((270.0 - f) + 270.0)
                elif f > i + 270 and f < i4 + 270:
                    radians = math.radians(270.0 - (f - 270.0))
                elif f > 90 - i4 and f < 90 - i:
                    radians = math.radians((90.0 - f) + 90.0)
                elif f > i + 90 and f < i4 + 90:
                    radians = math.radians(90.0 - (f - 90.0))
                else:
                    radians = math.radians(f)
        d = f2
        self.list_.append(int(math.sin(radians) * d))
        self.list_.append(int(d * math.cos(radians)))
        return copy.copy(self.list_)

    def transfrom3(self, f, f2):
        """Calculate the transformation of input angles to radians and perform
        trigonometric calculations.

        This method calculates the transformation of input angles to radians
        based on certain threshold values. It then performs trigonometric
        calculations using sine and cosine functions to determine the output
        values.

        Args:
            self: The object instance.
            f (float): The input angle in degrees.
            f2 (float): The input value for trigonometric calculations.

        Returns:
            list: A list containing the calculated values based on trigonometric
                functions.

        """
        radians = 0.0
        self.list_.clear()
        i = self.thresholdValue_2
        if f > 90 - i and f < i + 90:
            radians = math.radians(90.0)
        elif f > 270 - i and f < i + 270:
            radians = math.radians(270.0)
        else:
            i2 = self.thresholdValue_3
            if f > 180 - i2 and f < i2 + 180:
                radians = math.radians(180.0)
            elif f < i2 or f > 360 - i2:
                radians = math.radians(0.0)
            else:
                if f > i2:
                    i3 = self.thresholdValue_1
                    if f < 90 - i3:
                        radians = math.radians(90 - i3)
                i4 = self.thresholdValue_1
                if f > i4 + 90 and f < 180 - i2:
                    radians = math.radians(i4 + 90)
                elif f > i2 + 180 and f < 270 - i4:
                    radians = math.radians(i4 + 270)
                elif f > i4 + 270 and f < 360 - i2:
                    radians = math.radians(270 - i4)
                elif f > 270 - i4 and f < 270 - i:
                    radians = math.radians((270.0 - f) + 270.0)
                elif f > i + 270 and f < i4 + 270:
                    radians = math.radians(270.0 - (f - 270.0))
                else:
                    radians = math.radians(f)
        d = f2
        self.list_.append(int(math.sin(radians) * d))
        self.list_.append(int(d * math.cos(radians)))
        return copy.copy(self.list_)



from asyncio import Future
from typing import Any

import async_timeout


class MammotionFuture:
    """Create futures for each MQTT Message."""
    def __init__(self):
        self.fut: Future = Future()
        self.loop = self.fut.get_loop()

    def _resolve(self, item: bytes) -> None:
        if not self.fut.cancelled():
            self.fut.set_result(item)

    def resolve(self, item: bytes) -> None:
        self.loop.call_soon_threadsafe(self._resolve, item)

    async def async_get(self, timeout: float | int) -> bytes:
        try:
            async with async_timeout.timeout(timeout):
                return await self.fut
        finally:
            self.fut.cancel()
